/*
linphonec
Copyright (C) 2002  Florian Winterstein (flox@gmx.net)
Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "../oRTP/src/ortp.h"
/* to avoid conflict between config.h from oRTP and config.h from linphone */
#undef PACKAGE
#undef VERSION

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <sys/socket.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <sys/errno.h>
#include <string.h>
#include <signal.h>


#include "osphone.h"
#include "sip_handlers.h"
#include "ctype.h"

#include "../mediastreamer/msspeexdec.h"

extern char * return_audio_port(SdpContext *ctx,int i);
extern osphoneMain ospmain;

PayloadType lpc1015={
    PAYLOAD_AUDIO_PACKETIZED,
    8000,
    0,
    NULL,
    0,
    2400,
    "1015/8000/1"
};

PayloadType speex_nb={
    PAYLOAD_AUDIO_PACKETIZED,
    8000,
    0,
    NULL,
    0,
    15000,
    "speex/8000/1"
};

PayloadType speex_nb_lbr={
    PAYLOAD_AUDIO_PACKETIZED,
    8000,
    0,
    NULL,
    0,
    8000,
    "speex-lbr/8000/1"
};


/* table of different bandwidth for different connection type*/
const double bandwidths[5]=
{
    28800.0,  /*28k modem*/
    28800.0,  /*56k modem*/
    64000.0,  /*64k modem (numeris)*/
    128000.0, /*adsl or cable*/
    10e6,     /* ethernet */
};/* carefull: these are the upstream bandwidth !!!*/

int osp_main_loop(osphoneMain *opm)
{
    char input[256], ch;
    int i;
    
    ch = 0;

    while (ch != 'q')
    {
	fprintf(stdout, "\nCommand ? "); fflush(stdout);
	fgets(input, 255, stdin);
	ch = (char)tolower(input[0]);
	
	switch (ch)
	{
	    case 'r' :
		do_registration(NULL);
		break;

	    case 'c' :
		if (strlen(input)>2)
		    initiate_call(input+2);
		break;
		
	    case 's' :
		if (strlen(input)>2)
		{
		    i = atoi(input+2);
		    sprintf(input,"s%d",i);
		    if (psp_config_get_element(input)!=NULL)
			initiate_call(psp_config_get_element(input));
		}
		break;
		
	    case 'h' :
		hangup_call();	
		break;
		
	    case 'a' :
		answer_call();
		break;
		
	    case 'q' :
		printf("bye..\n");
		break;
		
	    default:
		fprintf(stdout, "\nHelp:\nr\tregister\nc <url>\tmake call\ns #\tshort dail\nh\thang up\na\tanswer call\nq\tquit\n");
		break;
	}
		
    }

    return 0;
}

/* init object */
int osp_main_init(osphoneMain *opm)
{
    memset(opm,0,sizeof(osphoneMain));
    osipua_init();
    ortp_init();
    ortp_set_debug_file("oRTP",NULL);
    rtp_profile_set_payload(&av_profile,115,&lpc1015);
	rtp_profile_set_payload(&av_profile,110,&speex_nb_lbr);
	rtp_profile_set_payload(&av_profile,111,&speex_nb);
	ms_init();
	ms_speex_codec_init();
	opm->codecs_conf.codecs=ms_codec_get_all_audio();
	opm->codecs_conf.audio_codecs=ms_codec_get_all_audio();
	opm->sound_conf.local_ring=g_strdup_printf("%s/%s",PACKAGE_SOUND_DIR,LOCAL_RING);
	opm->sound_conf.remote_ring=g_strdup_printf("%s/%s",PACKAGE_SOUND_DIR,REMOTE_RING);
    return 0;
}


/* error codes=
   -1: config file not found.
   -2: previous network interface not found
   -3: local loopback used
   -4: no networks interface founds
*/	
/* this functions loads linphone configuration from a config file. If the file does not exists, returns -1.*/
int osp_main_read_config(osphoneMain *opm)
{
    int err=0,i,retval=0;
    // net
    if (psp_config_get_element("local_addr")!=NULL)
    {
	opm->net_conf.local_addr=psp_config_get_element("local_addr");
    }
    if (psp_config_get_element("if_name")!=NULL)
    {
	opm->net_conf.if_name=psp_config_get_element("if_name");
    }
    if (psp_config_get_element("con_type")!=NULL)
    {
	opm->net_conf.con_type=atoi(psp_config_get_element("con_type"));
	opm->net_conf.bandwidth=bandwidths[(int)opm->net_conf.con_type];
    }
    // rtp
    if (psp_config_get_element("audio_rtp_port")!=NULL)
    {
	opm->rtp_conf.audio_rtp_port=atoi(psp_config_get_element("audio_rtp_port"));
    }
    if (psp_config_get_element("jitt_comp")!=NULL)
    {
	opm->rtp_conf.jitt_comp=atoi(psp_config_get_element("jitt_comp"));
    }
    // sound
    if (psp_config_get_element("driver_mode")!=NULL)
    {
	opm->sound_conf.driver_mode=atoi(psp_config_get_element("driver_mode"));
    }
    if (psp_config_get_element("rec_lev")!=NULL)
    {
	opm->sound_conf.rec_lev=atoi(psp_config_get_element("rec_lev"));
    }
    if (psp_config_get_element("play_lev")!=NULL)
    {
	opm->sound_conf.play_lev=atoi(psp_config_get_element("play_lev"));
    }
    if (psp_config_get_element("source")!=NULL)
    {
	opm->sound_conf.source=atoi(psp_config_get_element("source"));
    }
    if (psp_config_get_element("autokill")!=NULL)
    {
	opm->sound_conf.autokill=atoi(psp_config_get_element("autokill"));
    }
	
    // sip
    if (psp_config_get_element("sip_port")!=NULL)
    {
	opm->sip_conf.sip_port=atoi(psp_config_get_element("sip_port"));
    }
    else
	opm->sip_conf.sip_port=5060;
    if (psp_config_get_element("username")!=NULL)
    {
	opm->sip_conf.username=psp_config_get_element("username");
    }
    if (psp_config_get_element("hostname")!=NULL)
    {
	opm->sip_conf.hostname=psp_config_get_element("hostname");
    }
    if (psp_config_get_element("use_registrar")!=NULL)
    {
	opm->sip_conf.reg_conf.use_registrar=atoi(psp_config_get_element("use_registrar"));
    }
    else
	opm->sip_conf.reg_conf.use_registrar=0;
    if (psp_config_get_element("registrar")!=NULL)
    {
	opm->sip_conf.reg_conf.registrar=psp_config_get_element("registrar");
    }
    if (psp_config_get_element("reg_passwd")!=NULL)
    {
	opm->sip_conf.reg_conf.passwd=psp_config_get_element("reg_passwd");
    }
    else 
	opm->sip_conf.reg_conf.passwd=NULL; 
    if (psp_config_get_element("addr_of_rec")!=NULL)
    {
	opm->sip_conf.reg_conf.addr_of_rec=psp_config_get_element("addr_of_rec");
    }
    if (psp_config_get_element("reg_expires")!=NULL)
    {
	opm->sip_conf.reg_conf.expires=atoi(psp_config_get_element("reg_expires"));
    }
    if (psp_config_get_element("as_proxy")!=NULL)
    {
	opm->sip_conf.reg_conf.as_proxy=atoi(psp_config_get_element("as_proxy"));
    }
    if (psp_config_get_element("as_redirect")!=NULL)
    {
	opm->sip_conf.reg_conf.as_redirect=atoi(psp_config_get_element("as_redirect"));
    }
    if (psp_config_get_element("as_outbound")!=NULL)
    {
	opm->sip_conf.reg_conf.as_outbound=atoi(psp_config_get_element("as_outbound"));
    }

    /* set codec configuration*/
    osp_main_codecs_check(opm);
	
    /* check if saved network configuration is compatible with current one*/
    opm->net_conf.if_count=err=retrieve_if(opm);
    switch(err)
    {
	case 0:
	    retval=-2; 
	    osip_trace(TRACE_LEVEL0,("Warning: using local loopback network\n"));
	    opm->net_conf.local_addr="127.0.0.1";
	    /*strcpy(osp_conf->if_name,"lo");*/
	    break;
	case -1:
	    osip_trace(TRACE_LEVEL0,("ERROR: No network interface found.\n"));
	    retval=-4;
	    break;
	default:
	    retval=-2;
	    if (strcmp("lo",opm->net_conf.if_name)==0)
	    {	/* find first availlable interface and exits*/
		osip_trace(TRACE_LEVEL0,("Using %s interface with ip address %s\n",opm->net_conf.if_names[0],opm->net_conf.if_addr[0]));			
		opm->net_conf.local_addr=opm->net_conf.if_addr[0];
		opm->net_conf.if_sel=0;
		opm->net_conf.if_name=opm->net_conf.if_names[0];
		retval=0;
	    }
	    else
	    {   /* else try to find previously used network interface */
		for(i=0;i<err;i++)
		{
		    if (strcmp(opm->net_conf.if_name,opm->net_conf.if_names[i])==0)
		    {
			osip_trace(TRACE_LEVEL0,("Found %s interface with ip address %s\n",opm->net_conf.if_names[i],opm->net_conf.if_addr[i]));			
			opm->net_conf.local_addr=opm->net_conf.if_addr[i];
			opm->net_conf.if_sel=i;
			retval=0;
		    }
		}
		if (retval==-2)
		{
		    osip_trace(TRACE_LEVEL0,("Cannot find interface %s.\n",opm->net_conf.if_name));
		    osip_trace(TRACE_LEVEL0,("using %s interface with ip address %s\n",opm->net_conf.if_names[0],opm->net_conf.if_addr[0]));
		    opm->net_conf.local_addr=opm->net_conf.if_addr[0];
		    opm->net_conf.if_sel=0;	
		}
	    }
	    break;
    }	
    return(retval);
}



/* create the sip user agent and run it*/
int osp_main_start(osphoneMain *opm)
{
    char *prim_contact;
    int err,len;
    BodyHandler *sdph;
    OsipUA *ua;
	
	
    len=strlen(opm->net_conf.local_addr)+12+strlen(opm->sip_conf.username);
    prim_contact=g_malloc(len);
    if (opm->sip_conf.sip_port==5060)
	snprintf(prim_contact,len,"sip:%s@%s",opm->sip_conf.username,opm->net_conf.local_addr);
    else 
	snprintf(prim_contact,len,"sip:%s@%s:%i",opm->sip_conf.username,opm->net_conf.local_addr,opm->sip_conf.sip_port);
    ua=osip_ua_new();
    opm->ua=ua;
    osip_ua_set_contact(ua,prim_contact);
    g_free(prim_contact);
	
    osp_main_set_contact(opm);    /* set aliases according to current config*/
	
    err=osip_ua_signal_connect(ua,"INVITE_ACCEPTED",invite_accepted_cb);
    err=osip_ua_signal_connect(ua,"BYE",bye_cb);
    err=osip_ua_signal_connect(ua,"FAILLURE",faillure_cb);
    err=osip_ua_signal_connect(ua,"INVITE",invite_cb);
    err=osip_ua_signal_connect(ua,"INFORMATIVE",informative_cb);
  	
    /* create a sdp handler */
    opm->sdph=sdph=sdp_handler_new();
    sdp_handler_set_fcn_get_offer(SDP_HANDLER(sdph),generate_sdp);
    sdp_handler_set_fcn_get_audio_port(SDP_HANDLER(sdph),return_audio_port);
    /* setup the media properties */
    osp_main_apply_codec_change(opm);
    /* add the sdp handler to the ua */
    osip_ua_add_body_handler(ua,sdph);
	
    osp_main_set_registration_timer(opm);
    if (opm->sip_conf.reg_conf.use_registrar)
    {
	if (opm->sip_conf.reg_conf.registrar!=NULL)
	{
	    len=strlen(opm->sip_conf.reg_conf.registrar);
	    if (len!=0)
	    {
		//if (opm->sip_conf.reg_conf.as_redirect) osip_ua_set_registrar(opm->ua,tmp2,opm->sip_conf.reg_conf.passwd);
		if (opm->sip_conf.reg_conf.as_proxy) osip_ua_set_outbound_proxy(opm->ua,opm->sip_conf.reg_conf.registrar,opm->sip_conf.reg_conf.passwd);
	    }	
	    do_registration(NULL);
	}
    }
    return(0);
}

int osp_main_set_contact(osphoneMain *opm)
{
    char *contact;
    char *user,*domain;
	
    if (opm->ua==NULL) return(-EFAULT);
	
	
    contact=malloc(strlen(opm->sip_conf.username)+strlen(opm->sip_conf.hostname)+22);
    user=opm->sip_conf.username;
    domain=opm->sip_conf.hostname;

    if (opm->sip_conf.sip_port==5060)
	sprintf(contact,"<sip:%s@%s>",user,domain);	
    else sprintf(contact,"<sip:%s@%s:%i>",user,domain,opm->sip_conf.sip_port);
    osip_trace(TRACE_LEVEL0,("Adding %s to the list of alias.\n",contact));
    osip_ua_add_alias(opm->ua,contact);
    if (opm->sip_conf.contact!=NULL) free(opm->sip_conf.contact);
    opm->sip_conf.contact=contact;
    return 0;
}
		

void osp_main_set_registration_timer(osphoneMain *opm)
{
    osip_trace(TRACE_LEVEL0,("osp_main_set_registration_timer - does nothing at the moment!\n"));
}

void osp_main_remove_registration_timer(osphoneMain *opm)
{
    osip_trace(TRACE_LEVEL0,("osp_main_remove_registration_timer - does nothing at the moment!\n"));
}



void osp_main_apply_codec_change(osphoneMain *opm)
{
    MSCodecInfo *codec;
    GList *elem;
	
    osp_main_codecs_check(opm);
    if (opm->ua==NULL) return;
    sdp_handler_remove_audio_payloads(SDP_HANDLER(opm->sdph));
    sdp_handler_remove_video_payloads(SDP_HANDLER(opm->sdph));
    elem=opm->codecs_conf.audio_codecs;
    while(elem!=NULL)
    {
	codec=(MSCodecInfo*)(elem->data);
	if ((codec->is_selected)&&(codec->is_usable))
	{
	    osip_trace(TRACE_LEVEL0,("Added payload type %i %s\n", codec->pt, codec->description));
	    sdp_handler_add_support_for_audio_codec(SDP_HANDLER(opm->sdph),int2str(codec->pt),
						    sgetcopy("RTP/AVP"),
						    sgetcopy(codec->description));
	}
	elem=g_list_next(elem);
    }
    elem=opm->codecs_conf.video_codecs;
    while(elem!=NULL)
    {
	codec=(MSCodecInfo*)(elem->data);
	//osip_ua_add_profile(opm->ua,STREAM_VIDEO,codec->profile);
	elem=g_list_next(elem);
    }
	
}

/* test if codecs are usable compared to bandwidth. */
void osp_main_codecs_check(osphoneMain *opm)
{
    GList *elem;
    MSCodecInfo *codinfo;
	
    elem=opm->codecs_conf.codecs;
    while(elem!=NULL)
    {
	codinfo=(MSCodecInfo*)elem->data;
	codinfo->is_usable=ms_codec_is_usable(codinfo,opm->net_conf.bandwidth);
	elem=g_list_next(elem);
    }
    
}
	
/* stop all processes properly*/
void osp_main_stop(osphoneMain *opm)
{
    osip_ua_destroy(opm->ua);
    osp_main_remove_registration_timer(opm);
    opm->ua=NULL;
}


void osp_main_shutdown(osphoneMain *opm, int (*cb)(OsipDialog*,transaction_t*,sip_t *,void*) )
{
    OsipDialog *call;
    /* Before leaving linphone should unregisters itself from the registrar (if used)
       unregistration_ok_cb is called when the unregistration is successfull */
    if (opm->sip_conf.reg_conf.use_registrar)
    {
	osip_ua_signal_connect(opm->ua,"INFORMATIVE",cb);
	if (strcmp(opm->net_conf.if_name,"lo")!=0)
	{
	    RegistrationCtxt *ctx;
	    ctx=registration_ctxt_new();
	    registration_ctxt_set_registrar(ctx,opm->sip_conf.reg_conf.registrar);
	    registration_ctxt_set_password(ctx,opm->sip_conf.reg_conf.passwd);
	    registration_ctxt_set_address_of_record(ctx,opm->sip_conf.reg_conf.addr_of_rec);		
	    call=osip_dialog_new(opm->ua);
	    osip_dialog_unregister(call,ctx);
	    osip_dialog_release(call);
	}
    }
}
	


static int codec_cmp(gconstpointer codec, gconstpointer profile)
{
    MSCodecInfo *p= (MSCodecInfo*) codec;
    return (p->pt!=GPOINTER_TO_INT(profile)) ;
}

MSCodecInfo * osp_main_get_codec(osphoneMain *opm, gint payload_type)
{
    GList *l;
    l=g_list_find_custom(opm->codecs_conf.codecs,GINT_TO_POINTER(payload_type),codec_cmp);
    if (l==NULL)
    {
	osip_trace(TRACE_LEVEL0,("Cannot find codec with payload type %i\n",payload_type));
	return(NULL);
    }
    return((MSCodecInfo*)(l->data));
}

/* final exit: stops all threads for gracefull shutdown*/
void osp_main_exit(osphoneMain *opm)
{
    osipua_exit();
}


/* this is the retrieve_if function for FreeBSD. It uses getifaddrs system call instead of ioctl in order to get interface names and address.*/
/* Thanks 	to Wolfgang for this .*/
#ifdef HAVE_GETIFADDRS	
int retrieve_if(osphoneMain *opm)
{
    struct ifaddrs *ifp;
    int i;

    if(getifaddrs(&ifp) < 0) {
        return -1;
    }
    for(i = 0;
        ifp != NULL;
        ifp = ifp->ifa_next) {

        if(ifp->ifa_addr->sa_family != AF_INET) continue;

	osip_trace(TRACE_LEVEL0,("ifc %s %s\n",ifp->ifa_name,inet_ntoa(((struct sockaddr_in *)ifp->ifa_addr)->sin_addr)));
        strncpy(opm->net_conf.if_names[i],
                ifp->ifa_name,
                sizeof opm->net_conf.if_names[i]);
        strncpy(opm->net_conf.if_addr[i],
                inet_ntoa(((struct sockaddr_in *)ifp->ifa_addr)->sin_addr),
                sizeof opm->net_conf.if_addr[i]);
        ++i;
    }

    freeifaddrs(ifp);

    return i;
}
#endif

int retrieve_if(osphoneMain *opm)
{
    int sock,err,if_count,i,j=0,lo=0;
    struct ifconf netconf;
    char buffer[32*MAX_IF];
	
    netconf.ifc_len=32*MAX_IF;
    netconf.ifc_buf=buffer;
    sock=socket(PF_INET, SOCK_DGRAM,0);
    err=ioctl(sock,SIOCGIFCONF,&netconf);
    if (err<0) osip_trace(TRACE_LEVEL0,("Error in ioctl: %i.\n",errno));
    close(sock);
	
    if_count=netconf.ifc_len/32;
    osip_trace(TRACE_LEVEL0,("Found %i interfaces.\n",if_count));
    for (i=0;i<if_count;i++)
    {
	if (strcmp(netconf.ifc_req[i].ifr_name,"lo")==0) lo=1;

	strncpy(opm->net_conf.if_names[j],netconf.ifc_req[i].ifr_name,20);
	strncpy(opm->net_conf.if_addr[j],inet_ntoa(((struct sockaddr_in*)(&netconf.ifc_req[i].ifr_addr))->sin_addr),20);
	j++;
	
    }
    if (lo==0) return(-1);
    return(if_count);
}

int kill_sound_daemons()
{
    char command[100];
    int err;
    FILE *out;
    pid_t pid;
	
    /* try to kill all artsd*/
    snprintf(command,100,"ps -u %s |grep artsd",getenv("LOGNAME"));
    out=popen(command,"r");
    if (out!=NULL)
    {
	do{
	    err=fscanf(out,"%i",&pid);
	    if (err==1) kill(pid,SIGINT);
	}while(err==1);
	pclose(out);
    }
    /* do the same with esd*/
    snprintf(command,100,"ps -u %s |grep esd",getenv("LOGNAME"));
    out=popen(command,"r");
    if (out!=NULL)
    {
	do{
	    err=fscanf(out,"%i",&pid);
	    if (err==1) kill(pid,SIGINT);
	}while(err==1);
	pclose(out);
    }
    return(0);
}




/*try to open dsp. If failed, kill sound daemons if allowed by user, and try agin. If fail again, display the error message. Close it in success*/
int try_open_dsp()
{
	int err1,err2,retval=-1;
	
	err1=test_audio_dev(ospmain.sound_conf.dev_id);
	if (!err1)
	{
		if (ospmain.sound_conf.autokill)
		{
			printf("killing sound daemons.\n");
			kill_sound_daemons();
			sleep(1);   /* wait for sound daemons to be killed...*/
		}
		else
		{
			/* here we sould propose to kill the daemons*/
			/* to do: we required a blocking dialog box here : mutex ??*/
			return(-1);
		}
		err2=test_audio_dev(ospmain.sound_conf.dev_id);
		if (!err2)
		{
			
			return(-1);
		}
		else retval=err2;
	}
	else retval=err1;
	if (retval>0) ospmain.sound_conf.latency=retval/8;
	return(retval);
}
	

char *int2str(int number)
{
    gchar *numstr=g_malloc(10);
    snprintf(numstr,10,"%i",number);
    return numstr;
}

void print_usage()
{
    fprintf(stdout, "\n\
usage: linphonec [-c file] [-d level -l logfile]\n\
\n\
  -c  file             specify the  configuration file or use - ~/.linphonec\n\
  -d  level            be verbose. 0 is no output. 6 is all output\n\
  -l  logfile          specify the log file for your SIP phone\n\n");
}
