/***************************************************************************
                          linphone  - sipomatic.c
This is a test program for linphone. It acts as a sip server and answers to linphone's
call.
                             -------------------
    begin                : ven mar  30
    copyright            : (C) 2001 by Simon MORLAT
    email                : simon.morlat@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <unistd.h>
#include <signal.h>
#include "sipomatic.h"
#include <glib.h>


int run_cond=1;

Sipomatic sipomatic;

PayloadType lpc1015={
    PAYLOAD_AUDIO_PACKETIZED,
    8000,
    0,
    NULL,
    0,
    2400,
    "1015/8000/1"
};

PayloadType speex_nb={
    PAYLOAD_AUDIO_PACKETIZED,
    8000,
    0,
    NULL,
    0,
    15000,
    "speex/8000/1"
};

PayloadType speex_nb_lbr={
    PAYLOAD_AUDIO_PACKETIZED,
    8000,
    0,
    NULL,
    0,
    8000,
    "speex-lbr/8000/1"
};

static char *num2str(int a)
{
	char *p=smalloc(10);
	snprintf(p,10,"%i",a);
	return p;
}


void stop_handler(int signum)
{
	run_cond=0;
}

int invite_accepted_cb(OsipDialog *call,transaction_t *trn, sip_t *msg,void *p2)
{
	/* we should never be here since sipomatic is unable to to make invites*/
	return(0);
}

int bye_cb(OsipDialog *dialog,transaction_t *trn, sip_t *msg,void *p2)
{
	/* get the call */
	Call *call=(Call*)dialog->data;
	sipomatic_lock(&sipomatic);
	call->state=CALL_STATE_FINISHED;
	sipomatic_unlock(&sipomatic);
	return(0);
}

int invite_cb(OsipDialog *dialog,transaction_t *trn, sip_t *sipmsg,void *p2)
{
	/* we have just received an invite, so wait two seconds, then accept it*/
	char *p;
	Call *call;
	
	from_2char(sipmsg->from,&p);
	printf("Contacted by %s.\n",p);
	sfree(p);
	call=call_new(dialog);
	call->current_trn=trn;
	call->root=&sipomatic;
	sipomatic_lock(&sipomatic);
	sipomatic.calls=g_list_append(sipomatic.calls,call);
	sipomatic_unlock(&sipomatic);
	return(0);
}

void call_accept(Call *call)
{
	char *payload,*remport,*locport,*remaddr;
	SdpContext *ctx;
	int status;
	sdp_t *local,*remote;
	OsipDialog *dialog=call->dialog;
	
	printf("Answering...\n");
	ctx=SDP_CONTEXT(osip_dialog_get_body_context(dialog,"application/sdp",0));
	status=sdp_context_get_negociation_status(ctx);
	if (status!=200){
		printf("Error during sdp negociation, cannot accept call.\n");
		osip_dialog_respond(dialog,call->current_trn,415);
		call->state=CALL_STATE_FINISHED;
		return;
	}
	osip_dialog_accept_invite(dialog,call->current_trn);
	local=ctx->localneg;
	remote=ctx->remote;
	payload=sdp_m_payload_get(local,0,0);
	remaddr=sdp_c_addr_get(remote,-1,0);
	if (remaddr==NULL){
		printf("Could not get remote address !\n");
		exit(0);
	}
	remport=sdp_m_port_get(remote,0);
	locport=sdp_m_port_get(local,0);
 	call->audio_stream=audio_stream_start_with_files(atoi(locport),remaddr,atoi(remport),
			atoi(payload),20,call->root->file_path,"/dev/null");
	g_timer_reset(call->timer);
	call->state=CALL_STATE_RUNNING;
}


int faillure_cb(OsipDialog *call,transaction_t *trn, sip_t *msg,void *p2)
{
	return(0);
}


G_LOCK_DEFINE_STATIC(port_lock);

char *get_audio_port(SdpContext *ctx,int i)
{
	static guint16 port=6000;
	char *p;
	G_LOCK(port_lock);
	p=smalloc(10);
	sprintf(p,"%i",port);
	port+=4;
	if (port<6000) port=6000;
	G_UNLOCK(port_lock);
	return p;
}

void sipomatic_init(Sipomatic *obj)
{
	char *contact;
	MSCodecInfo *info;
	GList *elem;
	BodyHandler *sdph;
	contact=getenv("SIPOMATIC_URL");
	if (contact==NULL) contact="sip:robot@127.0.0.1:5064";
	g_message("Starting using url %s",contact);
	obj->audio_codecs=ms_codec_get_all_audio();
	obj->lock=g_mutex_new();
	obj->calls=NULL;
	obj->acceptance_time=5;
	obj->max_call_time=14;
	obj->file_path=g_strdup_printf("%s/%s",PACKAGE_SOUND_DIR,ANNOUCE_FILE);
	
	/* create a user agent */
	obj->ua=osip_ua_new();
	osip_ua_set_contact(obj->ua,contact);
	osip_ua_signal_connect(obj->ua,"INVITE_ACCEPTED",invite_accepted_cb);
  	osip_ua_signal_connect(obj->ua,"BYE",bye_cb);
  	osip_ua_signal_connect(obj->ua,"FAILLURE",faillure_cb);
  	osip_ua_signal_connect(obj->ua,"INVITE",invite_cb);
	/* add sdp capabilities to the user agent */
	sdph=sdp_handler_new();
	osip_ua_add_body_handler(obj->ua,sdph);
	elem=obj->audio_codecs;
	while(elem!=NULL){
		info=(MSCodecInfo*) elem->data;
		sdp_handler_add_support_for_audio_codec(SDP_HANDLER(sdph),
					num2str(info->pt),sgetcopy("RTP/AVP"),sgetcopy(info->description));
		elem=g_list_next(elem);
	}
	sdp_handler_set_fcn_get_audio_port(SDP_HANDLER(sdph),get_audio_port);
	
}

void sipomatic_uninit(Sipomatic *obj)
{
	g_mutex_free(obj->lock);
	osip_ua_destroy(obj->ua);
}

void sipomatic_iterate(Sipomatic *obj)
{
	GList *elem;
	Call *call;
	gdouble time;
	sipomatic_lock(obj);
	elem=obj->calls;
	while(elem!=NULL){
		call=(Call*)elem->data;
		time=g_timer_elapsed(call->timer,NULL);
		switch(call->state){
			case CALL_STATE_INIT:
				if (time>obj->acceptance_time){
					call_accept(call);
				}
			break;
			case CALL_STATE_RUNNING:
				if (time>obj->max_call_time){
					call_release(call);
					elem=obj->calls=g_list_remove(obj->calls,call);
					call_destroy(call);
				}
			break;
			case CALL_STATE_FINISHED:
				elem=obj->calls=g_list_remove(obj->calls,call);
				call_destroy(call);
			break;
		}
		elem=g_list_next(elem);
	}
	sipomatic_unlock(obj);
}

Call * call_new(OsipDialog *dialog)
{
	Call *obj=g_new(Call,1);
	obj->timer=g_timer_new();
	obj->dialog=dialog;
	dialog->data=(void*)obj;
	g_timer_start(obj->timer);
	obj->audio_stream=NULL;
	obj->state=CALL_STATE_INIT;
	return obj;
}

void call_release(Call *call)
{
	osip_dialog_bye(call->dialog);
	call->state=CALL_STATE_FINISHED;
}
void call_destroy(Call *obj)
{
	if (obj->audio_stream!=NULL) audio_stream_stop(obj->audio_stream);
	g_timer_destroy(obj->timer);
	g_free(obj);
}

extern OsipManager *def_manager;

int main(int argc, char *argv[])
{
	int presence=200;
	
	
	if (argc==2) presence=atoi(argv[1]);
	
	signal(SIGINT,stop_handler);
	ms_init();
	ms_speex_codec_init();
	ortp_init();
	ortp_set_debug_file("oRTP",NULL);
	rtp_profile_set_payload(&av_profile,115,&lpc1015);
	rtp_profile_set_payload(&av_profile,110,&speex_nb);
	rtp_profile_set_payload(&av_profile,111,&speex_nb_lbr);
	TRACE_INITIALIZE(TRACE_LEVEL6,stdout);
	osipua_init();
	osip_manager_set_send_port(def_manager,5070);
	sipomatic_init(&sipomatic);
	
	/*
	osip_ua_set_presence_mode(ua,presence);
	osip_ua_set_presence_delay(ua,60);
	osip_ua_set_presence_contact_url(ua,"sip:toto@free.fr");
	*/
	
	//snprintf(snd_path,100,"%s%s",PACKAGE_DATA_DIR,"/sounds/hello.wav");
	
	while (run_cond){
		sipomatic_iterate(&sipomatic);
		usleep(20000);
	}
	
	return(0);
}

