/*
linphone
Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "../oRTP/src/ortp.h"
/* to avoid conflict between config.h from oRTP and config.h from linphone */
#undef PACKAGE
#undef VERSION

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <stdlib.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <sys/socket.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <sys/errno.h>
#include <string.h>
#include <signal.h>


#include "LinphoneMain.h"
#include "sip_handlers.h"
#include "gui_utils.h"
#include <mediastream.h>

extern char * return_audio_port(SdpContext *ctx,int i);


PayloadType lpc1015={
    PAYLOAD_AUDIO_PACKETIZED,
    8000,
    0,
    NULL,
    0,
    2400,
    "1015/8000/1"
};

PayloadType speex_nb={
    PAYLOAD_AUDIO_PACKETIZED,
    8000,
    0,
    NULL,
    0,
    15000,
    "speex/8000/1"
};

PayloadType speex_nb_lbr={
    PAYLOAD_AUDIO_PACKETIZED,
    8000,
    0,
    NULL,
    0,
    8000,
    "speex-lbr/8000/1"
};

/* table of different bandwidth for different connection type*/
const double bandwidths[5]=
{
	28800.0,	/*28k modem*/
	28800.0, /*56k modem*/
	64000.0, /*64k modem (numeris)*/
	128000.0,	/*adsl or cable*/
	10e6, /* ethernet */
};/* carefull: these are the upstream bandwidth !!!*/


static int codec_cmp(gconstpointer codec, gconstpointer profile)
{
	MSCodecInfo *p= (MSCodecInfo*) codec;
	return (p->pt!=GPOINTER_TO_INT(profile)) ;
}


/* init object */
void linphone_main_init(LinphoneMain *lpm)
{
	memset(lpm,0,sizeof(LinphoneMain));
	TRACE_INITIALIZE(TRACE_LEVEL5,stdout);
	osipua_init();
	ortp_init();
	ortp_set_debug_file("oRTP",NULL);
	rtp_profile_set_payload(&av_profile,115,&lpc1015);
	rtp_profile_set_payload(&av_profile,110,&speex_nb);
	rtp_profile_set_payload(&av_profile,111,&speex_nb_lbr);
	ms_init();
	ms_speex_codec_init();
	lpm->codecs_conf.codecs=ms_codec_get_all_audio();
	//printf("%i codecs found.\n",g_list_length(lpm->codecs_conf.codecs));
}
/* if there is no config file, use this functions to set default parameters*/
/* error codes=
	-3: local loopback used
	-4: no networks interface founds
	*/
int linphone_main_first_startup(LinphoneMain *lpm)
{
	int err,retval=0,index=0;
	char *p;
	
	/* set network configuration*/
	lpm->net_conf.if_count=err=retrieve_if(lpm);
	switch(err)
	{
		case 1:
			printf("Warning: using local loopback network->\n");
			lpm->net_conf.local_addr="127.0.0.1";
			lpm->net_conf.if_name="lo";
			retval=-3;
			break;
		case -1:
			printf("ERROR: No network interface found.\n");
			retval=-4;
			break;
		default:
			if (strcmp(lpm->net_conf.if_names[0],"lo")==0) index++;
			printf("using %s interface with ip address %s\n",lpm->net_conf.if_names[index],lpm->net_conf.if_addr[index]);
			lpm->net_conf.local_addr=lpm->net_conf.if_addr[index];
			lpm->net_conf.if_sel=index;
			lpm->net_conf.if_name=lpm->net_conf.if_names[index];
			break;
	}
	lpm->net_conf.con_type=1;   /* default to 56k modem*/
	lpm->net_conf.bandwidth=bandwidths[(int)lpm->net_conf.con_type];
	/* set sip configuration*/
	lpm->sip_conf.sip_port=5060;
	p=getenv("LOGNAME");
	if (p==NULL) p="NO_USER_NAME";
	lpm->sip_conf.username=sgetcopy(p);
	p= getenv("HOSTNAME");
	if (p==NULL) p="NO_HOST_NAME";
	lpm->sip_conf.hostname=sgetcopy(p);
	lpm->sip_conf.reg_conf.expires=REG_EXPIRES;  /* 15 minutes*/
		
	/*set rtp configuration*/
	lpm->rtp_conf.jitt_comp=150;
	lpm->rtp_conf.audio_rtp_port=7078;


	/* set codec configuration*/
	lpm->codecs_conf.audio_codecs=ms_codec_get_all_audio();
	linphone_main_codecs_check(lpm);
	
	/* set default sound driver */
#ifdef ALSA
	lpm->sound_conf.driver_mode=1;
#else
	lpm->sound_conf.driver_mode=0;
#endif
	if (err!=0) printf("Cannot initialise sound support.\n");
	/* set recording source*/
	lpm->sound_conf.source='m';
	
	/* set default levels*/
	lpm->sound_conf.rec_lev=75;
	lpm->sound_conf.play_lev=75;
	
	
	
	lpm->sound_conf.autokill=0;
	lpm->sound_conf.local_ring=g_strdup_printf("%s/%s",PACKAGE_SOUND_DIR,LOCAL_RING);
	lpm->sound_conf.remote_ring=g_strdup_printf("%s/%s",PACKAGE_SOUND_DIR,REMOTE_RING);
	
	return(retval);
}


/* error codes=
	-1: config file not found.
	-2: previous network interface not found
	-3: local loopback used
	-4: no networks interface founds
	*/	
/* this functions loads linphone configuration from a config file. If the file does not exists, returns -1.*/
int linphone_main_read_config(LinphoneMain *lpm)
{
	int err=0,i,retval=0;
	int test,len,tmp;
	GList *list,*elem=NULL;
	char **vector;
	MSCodecInfo *codec;
	gchar *ab_entry=g_malloc(100);
	
	/* do a test to find if there is a config file*/
	test=gnome_config_has_section("/linphone/global");
	if (!test) return(-1);
	/* get version of this config file*/
	test=gnome_config_get_int("/linphone/version/version_id");
	if (test==0)
	{
		/* version not found, old linphone*/
		printf("Old config file found: deleted.\n");
		gnome_config_drop_all();
		gnome_config_clean_file("/linphone");
		gnome_config_sync();
		return(-1);
	}
	
	/* there is a config file, so read it*/
	/*network config*/
	lpm->net_conf.local_addr=gnome_config_get_string("/linphone/global/local_addr");
	lpm->net_conf.if_name=gnome_config_get_string("/linphone/global/if_name");
	lpm->net_conf.con_type=gnome_config_get_int("/linphone/global/con_type");
	lpm->net_conf.bandwidth=bandwidths[(int)lpm->net_conf.con_type];
	/* rtp config*/
	lpm->rtp_conf.audio_rtp_port=gnome_config_get_int("/linphone/rtp/audio_rtp_port");
	lpm->rtp_conf.jitt_comp=gnome_config_get_int("/linphone/rtp/jitt_comp");
	if (lpm->rtp_conf.jitt_comp<20){
		/* probably using old values expressed in number of packets. */
		lpm->rtp_conf.jitt_comp=150;
	}
	/* sound config*/
	lpm->sound_conf.driver_mode=gnome_config_get_int("/linphone/audio/driver_mode");
	lpm->sound_conf.rec_lev=gnome_config_get_int("/linphone/audio/rec_lev");
	lpm->sound_conf.play_lev=gnome_config_get_int("/linphone/audio/play_lev");
	lpm->sound_conf.source=gnome_config_get_int("/linphone/audio/source");
	lpm->sound_conf.autokill=gnome_config_get_int("/linphone/audio/autokill");
	lpm->sound_conf.local_ring=gnome_config_get_string("/linphone/audio/local_ring");
	lpm->sound_conf.remote_ring=gnome_config_get_string("/linphone/audio/remote_ring");
	if (lpm->sound_conf.local_ring==NULL)
		lpm->sound_conf.local_ring=g_strdup_printf("%s/%s",PACKAGE_SOUND_DIR,LOCAL_RING);
	if (lpm->sound_conf.remote_ring==NULL) 
		lpm->sound_conf.remote_ring=g_strdup_printf("%s/%s",PACKAGE_SOUND_DIR,REMOTE_RING);
	
	/* sip config*/
	lpm->sip_conf.sip_port=gnome_config_get_int("/linphone/sip/sip_port");	
	/*temp code for sip config*/
	lpm->sip_conf.username=gnome_config_get_string("/linphone/sip/username");
	if (lpm->sip_conf.username==NULL) lpm->sip_conf.username=sgetcopy("toto");
	lpm->sip_conf.hostname=gnome_config_get_string("/linphone/sip/hostname");
	if (lpm->sip_conf.hostname==NULL) lpm->sip_conf.hostname=sgetcopy("at.home");
	lpm->sip_conf.reg_conf.use_registrar=gnome_config_get_int("/linphone/sip/use_registrar");
	lpm->sip_conf.reg_conf.registrar=gnome_config_get_string("/linphone/sip/registrar");	
	lpm->sip_conf.reg_conf.passwd= gnome_config_get_string("/linphone/sip/reg_passwd");
	lpm->sip_conf.reg_conf.addr_of_rec= gnome_config_get_string("/linphone/sip/addr_of_rec");
	lpm->sip_conf.reg_conf.expires= gnome_config_get_int("/linphone/sip/reg_expires");
	if (lpm->sip_conf.reg_conf.expires==0)  lpm->sip_conf.reg_conf.expires=REG_EXPIRES;
	lpm->sip_conf.reg_conf.as_proxy= (unsigned int) gnome_config_get_int("/linphone/sip/as_proxy");
	lpm->sip_conf.reg_conf.as_redirect= (unsigned int) gnome_config_get_int("/linphone/sip/as_redirect");
	lpm->sip_conf.reg_conf.as_outbound= (unsigned int) gnome_config_get_int("/linphone/sip/as_outbound");
	
	
	
	/* get codecs config*/
	gnome_config_get_vector("/linphone/codecs/audio_codecs",&len,&vector);
	if (len==0) printf("No codec list in config file!\n");
	list=NULL;
	for (i=0;i<len;i++)
	{
		tmp=atoi(vector[i]);
		codec=linphone_main_get_codec(lpm,tmp & 0xFF);
		if (codec!=NULL)
		{
			list=g_list_append(list,codec);
			codec->is_selected=tmp>>8;
		}
		else {
			if ((tmp & 0xFF )==4){
				/* this is the old payload type for lpc10-15, but now it is 115. Change this */
				codec=linphone_main_get_codec(lpm,115);
				list=g_list_append(list,codec);
			}
		}
	}
	/* append to the end of the list codecs that are present in linphone but not in the conf
	file */
	elem=lpm->codecs_conf.codecs;
	while (elem!=NULL){
		codec=(MSCodecInfo*)(elem->data);
		if (g_list_find_custom(list,GINT_TO_POINTER(codec->pt),codec_cmp)==NULL){
			g_message("Added new codec %s - %s",MS_FILTER_INFO(codec)->name,codec->description);
			list=g_list_append(list,codec);
		}
		elem=g_list_next(elem);
	}
	
	lpm->codecs_conf.audio_codecs=list;
	
	/* get address book entries*/
	len= gnome_config_get_int("/linphone/address_book/entry_count");
	if (len!=-1)
	{
		for (tmp=0;tmp<len;tmp++)
		{
			snprintf(ab_entry,100,"/linphone/address_book/entry%i",tmp);
			lpm->gui_conf.address_list=g_list_append(lpm->gui_conf.address_list,(gpointer)gnome_config_get_string(ab_entry));
		}
	}
	
	/* set codec configuration*/
	linphone_main_codecs_check(lpm);
	
	/* check if saved network configuration is compatible with current one*/
	lpm->net_conf.if_count=err=retrieve_if(lpm);
	switch(err)
	{
		case 0:
			retval=-2;
			printf("Warning: using local loopback network\n");
			lpm->net_conf.local_addr="127.0.0.1";
			/*strcpy(linphone_conf->if_name,"lo");*/
			break;
		case -1:
			printf("ERROR: No network interface found.\n");
			retval=-4;
			break;
		default:
			retval=-2;
			if (strcmp("lo",lpm->net_conf.if_name)==0)
			{	/* find first availlable interface and exits*/
				printf("Using %s interface with ip address %s\n",lpm->net_conf.if_names[0],lpm->net_conf.if_addr[0]);			
				lpm->net_conf.local_addr=lpm->net_conf.if_addr[0];
				lpm->net_conf.if_sel=0;
				lpm->net_conf.if_name=lpm->net_conf.if_names[0];
				retval=0;
			}
			else
			{   /* else try to find previously used network interface */
				for(i=0;i<err;i++)
				{
					if (strcmp(lpm->net_conf.if_name,lpm->net_conf.if_names[i])==0)
					{
						printf("Found %s interface with ip address %s\n",lpm->net_conf.if_names[i],lpm->net_conf.if_addr[i]);			
						lpm->net_conf.local_addr=lpm->net_conf.if_addr[i];
						lpm->net_conf.if_sel=i;
						retval=0;
					}
				}
				if (retval==-2)
				{
					printf("Cannot find interface %s.\n",lpm->net_conf.if_name);
					printf("using %s interface with ip address %s\n",lpm->net_conf.if_names[0],lpm->net_conf.if_addr[0]);
					lpm->net_conf.local_addr=lpm->net_conf.if_addr[0];
					lpm->net_conf.if_sel=0;	
				}
			}
			break;
	}	
	return(retval);
}

/* save current linphone configuration*/
int linphone_main_save_config(LinphoneMain *lpm)
{
	int len,i,tmp;
	char **vector;
	char *table;
	MSCodecInfo *codec;
	GList *elem;
	gchar *ab_entry=g_malloc(100);

	gnome_config_set_int("/linphone/version/version_id",INT_VERSION);
	gnome_config_set_string("/linphone/global/local_addr",lpm->net_conf.local_addr);
	gnome_config_set_string("/linphone/global/if_name",lpm->net_conf.if_name);
	gnome_config_set_int("/linphone/global/con_type",lpm->net_conf.con_type);
	gnome_config_set_int("/linphone/rtp/audio_rtp_port",lpm->rtp_conf.audio_rtp_port);
	gnome_config_set_int("/linphone/rtp/jitt_comp",lpm->rtp_conf.jitt_comp);
	gnome_config_set_int("/linphone/audio/driver_mode",lpm->sound_conf.driver_mode);
	gnome_config_set_int("/linphone/audio/rec_lev",lpm->sound_conf.rec_lev);
	gnome_config_set_int("/linphone/audio/play_lev",lpm->sound_conf.play_lev);
	gnome_config_set_int("/linphone/audio/source",lpm->sound_conf.source);
	gnome_config_set_int("/linphone/audio/autokill",lpm->sound_conf.autokill);
	gnome_config_set_string("/linphone/audio/local_ring",lpm->sound_conf.local_ring);
	gnome_config_set_string("/linphone/audio/remote_ring",lpm->sound_conf.remote_ring);
	gnome_config_set_int("/linphone/sip/sip_port",lpm->sip_conf.sip_port);
	gnome_config_set_int("/linphone/sip/use_registrar",lpm->sip_conf.reg_conf.use_registrar);
	gnome_config_set_string("/linphone/sip/username",lpm->sip_conf.username);
	gnome_config_set_string("/linphone/sip/hostname",lpm->sip_conf.hostname);
	if (lpm->sip_conf.reg_conf.registrar==NULL) gnome_config_set_string("/linphone/sip/registrar","");
	else gnome_config_set_string("/linphone/sip/registrar",lpm->sip_conf.reg_conf.registrar);
	if (lpm->sip_conf.reg_conf.passwd==NULL) gnome_config_set_string("/linphone/sip/reg_passwd","");
	else gnome_config_set_string("/linphone/sip/reg_passwd",lpm->sip_conf.reg_conf.passwd);
	if (lpm->sip_conf.reg_conf.addr_of_rec==NULL) gnome_config_set_string("/linphone/sip/addr_of_rec","");
	else gnome_config_set_string("/linphone/sip/addr_of_rec",lpm->sip_conf.reg_conf.addr_of_rec);
	gnome_config_set_int("/linphone/sip/reg_expires",lpm->sip_conf.reg_conf.expires);
	gnome_config_set_int("/linphone/sip/as_proxy",lpm->sip_conf.reg_conf.as_proxy);
	gnome_config_set_int("/linphone/sip/as_redirect",lpm->sip_conf.reg_conf.as_redirect);
	gnome_config_set_int("/linphone/sip/as_outbound",lpm->sip_conf.reg_conf.as_outbound);
	
	/* need to set record codecs list*/
	len=g_list_length(lpm->codecs_conf.audio_codecs);
	vector=g_malloc(len*sizeof(char*));
	table=(char*)g_malloc(len*8);
	for (i=0;i<len;i++)
	{
		codec=(MSCodecInfo*)g_list_nth_data(lpm->codecs_conf.audio_codecs,i);
		tmp=codec->pt | (codec->is_selected << 8);   /* we add a special bit to tell if user wants to use it*/
		snprintf(&table[8*i],7,"%i",tmp);
		vector[i]=&table[8*i];
	}
	gnome_config_set_vector("/linphone/codecs/audio_codecs",len,vector);
	/* record address book entries*/
	len=0;
	elem=lpm->gui_conf.address_list;
	while(elem!=NULL)
	{
		snprintf(ab_entry,100,"/linphone/address_book/entry%i",len);
		gnome_config_set_string(ab_entry,(gchar*)(elem->data));
		elem=g_list_next(elem);
		len++;
	}
	gnome_config_set_int("/linphone/address_book/entry_count",len);
	
	gnome_config_sync ();
	g_free(vector);
	g_free(table);
	g_free(ab_entry);
	return(0);
}



/* create the sip user agent and run it*/
int linphone_main_start(LinphoneMain *lpm)
{
	char *prim_contact;
	int err,len;
	BodyHandler *sdph;
	OsipUA *ua;
	
	
	
	/* initialize the SIP stuff */
	len=strlen(lpm->net_conf.local_addr)+12+strlen(lpm->sip_conf.username);
	prim_contact=g_malloc(len);
	if (lpm->sip_conf.sip_port==5060)
		snprintf(prim_contact,len,"sip:%s@%s",lpm->sip_conf.username,lpm->net_conf.local_addr);
	else 
		snprintf(prim_contact,len,"sip:%s@%s:%i",lpm->sip_conf.username,lpm->net_conf.local_addr,lpm->sip_conf.sip_port);
	ua=osip_ua_new();
	lpm->ua=ua;
	osip_ua_set_contact(ua,prim_contact);
	g_free(prim_contact);
	
	linphone_main_set_contact(lpm);    /* set aliases according to current config*/
	
  	err=osip_ua_signal_connect(ua,"INVITE_ACCEPTED",invite_accepted_cb);
  	err=osip_ua_signal_connect(ua,"BYE",bye_cb);
  	err=osip_ua_signal_connect(ua,"FAILLURE",faillure_cb);
  	err=osip_ua_signal_connect(ua,"INVITE",invite_cb);
  	err=osip_ua_signal_connect(ua,"INFORMATIVE",informative_cb);
  	
	/* create a sdp handler */
	lpm->sdph=sdph=sdp_handler_new();
	sdp_handler_set_fcn_get_offer(SDP_HANDLER(sdph),generate_sdp);
	sdp_handler_set_fcn_get_audio_port(SDP_HANDLER(sdph),return_audio_port);
	/* setup the media properties */
	linphone_main_apply_codec_change(lpm);
	/* add the sdp handler to the ua */
	osip_ua_add_body_handler(ua,sdph);
	
	linphone_main_set_registration_timer(lpm);
	if (lpm->sip_conf.reg_conf.use_registrar)
	{
		if (lpm->sip_conf.reg_conf.registrar!=NULL)
		{
			len=strlen(lpm->sip_conf.reg_conf.registrar);
			if (len!=0)
			{
				//if (lpm->sip_conf.reg_conf.as_redirect) osip_ua_set_registrar(lpm->ua,tmp2,lpm->sip_conf.reg_conf.passwd);
				if (lpm->sip_conf.reg_conf.as_proxy) osip_ua_set_outbound_proxy(lpm->ua,lpm->sip_conf.reg_conf.registrar,lpm->sip_conf.reg_conf.passwd);
			}	
			do_registration(NULL);
		}
	}
	/* apply audio configuration */
	ms_oss_set_rlevel(&oss_mgr,lpm->sound_conf.dev_id,lpm->sound_conf.rec_lev);
	ms_oss_set_plevel(&oss_mgr,lpm->sound_conf.dev_id,lpm->sound_conf.play_lev);
	ms_oss_set_source(&oss_mgr,lpm->sound_conf.dev_id,lpm->sound_conf.source);
	
	return(0);
}

int linphone_main_set_contact(LinphoneMain *lpm)
{
	char *contact;
	char *user,*domain;
	
	if (lpm->ua==NULL) return(-EFAULT);
	
	
 	contact=malloc(strlen(lpm->sip_conf.username)+strlen(lpm->sip_conf.hostname)+22);
 	user=lpm->sip_conf.username;
 	domain=lpm->sip_conf.hostname;

	if (lpm->sip_conf.sip_port==5060)
		sprintf(contact,"<sip:%s@%s>",user,domain);	
	else sprintf(contact,"<sip:%s@%s:%i>",user,domain,lpm->sip_conf.sip_port);
	printf("Adding %s to the list of alias.\n",contact);
	osip_ua_add_alias(lpm->ua,contact);
	if (lpm->sip_conf.contact!=NULL) free(lpm->sip_conf.contact);
	lpm->sip_conf.contact=contact;
	return 0;
}
		

void linphone_main_set_registration_timer(LinphoneMain *lpm)
{
	/* set timer for redirect server registration*/
	if (lpm->sip_conf.reg_conf.use_registrar)
	{
		if (lpm->sip_conf.reg_conf.expires!=0)
		{
			if ( lpm->gui_conf.timer_id!=0) linphone_main_remove_registration_timer(lpm);
			lpm->gui_conf.timer_id=gtk_timeout_add( lpm->sip_conf.reg_conf.expires*1000,do_registration,NULL);
			printf("Setting timer for %i seconds.\n", lpm->sip_conf.reg_conf.expires);
		}
		else printf("Timer not set because of null timeval.\n");
	}
}

void linphone_main_remove_registration_timer(LinphoneMain *lpm)
{
	/* set timer for redirect server registration*/
	if (lpm->gui_conf.timer_id!=0)
	{
		gtk_timeout_remove(lpm->gui_conf.timer_id);
	}
}



void linphone_main_apply_codec_change(LinphoneMain *lpm)
{
	MSCodecInfo *codec;
	GList *elem;
	
	linphone_main_codecs_check(lpm);
	if (lpm->ua==NULL) return;
	sdp_handler_remove_audio_payloads(SDP_HANDLER(lpm->sdph));
	sdp_handler_remove_video_payloads(SDP_HANDLER(lpm->sdph));
	elem=lpm->codecs_conf.audio_codecs;
	while(elem!=NULL)
	{
		codec=(MSCodecInfo*)(elem->data);
		if ((codec->is_selected)&&(codec->is_usable))
		{
			g_message("Added payload type to SDP properties %i %s", codec->pt, codec->description);
			sdp_handler_add_support_for_audio_codec(SDP_HANDLER(lpm->sdph),int2str(codec->pt),
														sgetcopy("RTP/AVP"),
														sgetcopy(codec->description));
		}
		elem=g_list_next(elem);
	}
	elem=lpm->codecs_conf.video_codecs;
	while(elem!=NULL)
	{
		codec=(MSCodecInfo*)(elem->data);
		//osip_ua_add_profile(lpm->ua,STREAM_VIDEO,codec->profile);
		elem=g_list_next(elem);
	}
	
}

/* test if codecs are usable compared to bandwidth. */
void linphone_main_codecs_check(LinphoneMain *lpm)
{
	GList *elem;
	MSCodecInfo *codinfo;
	
	elem=lpm->codecs_conf.codecs;
	while(elem!=NULL)
	{
		codinfo=(MSCodecInfo*)elem->data;
		codinfo->is_usable=ms_codec_is_usable(codinfo,lpm->net_conf.bandwidth);
		elem=g_list_next(elem);
	}
}
	
/* stop all processes properly*/
void linphone_main_stop(LinphoneMain *lpm)
{
	osip_ua_destroy(lpm->ua);
	linphone_main_remove_registration_timer(lpm);
	lpm->ua=NULL;
}


void linphone_main_shutdown(LinphoneMain *lpm, int (*cb)(OsipDialog*,transaction_t*,sip_t *,void*) )
{
	OsipDialog *call;
	/* Before leaving linphone should unregisters itself from the registrar (if used)
	unregistration_ok_cb is called when the unregistration is successfull */
	if (lpm->sip_conf.reg_conf.use_registrar)
	{
		osip_ua_signal_connect(lpm->ua,"INFORMATIVE",cb);
		if (strcmp(lpm->net_conf.if_name,"lo")!=0)
		{
			RegistrationCtxt *ctx;
			ctx=registration_ctxt_new();
			registration_ctxt_set_registrar(ctx,lpm->sip_conf.reg_conf.registrar);
			registration_ctxt_set_password(ctx,lpm->sip_conf.reg_conf.passwd);
			registration_ctxt_set_address_of_record(ctx,lpm->sip_conf.reg_conf.addr_of_rec);		
			call=osip_dialog_new(lpm->ua);
			osip_dialog_unregister(call,ctx);
			osip_dialog_release(call);
		}
	}
}
	


MSCodecInfo * linphone_main_get_codec(LinphoneMain *lpm, gint payload_type)
{
	GList *l;
	l=g_list_find_custom(lpm->codecs_conf.codecs,GINT_TO_POINTER(payload_type),codec_cmp);
	if (l==NULL)
	{
		printf("Cannot find codec with payload type %i\n",payload_type);
		return(NULL);
	}
	return((MSCodecInfo*)(l->data));
}

/* final exit: stops all threads for gracefull shutdown*/
void linphone_main_exit(LinphoneMain *lpm)
{
	osipua_exit();
}

/* this is the retrieve_if function for FreeBSD. It uses getifaddrs system call instead of ioctl in order to get interface names and address.*/
/* Thanks 	to Wolfgang for this .*/
#ifdef HAVE_GETIFADDRS	
int retrieve_if(LinphoneMain *lpm)
{
    struct ifaddrs *ifp;
    int i;

    if(getifaddrs(&ifp) < 0) {
        return -1;
    }
    for(i = 0;
        ifp != NULL;
        ifp = ifp->ifa_next) {

        if(ifp->ifa_addr->sa_family != AF_INET) continue;

printf("ifc %s %s\n",
ifp->ifa_name,
inet_ntoa(
    ((struct sockaddr_in *)ifp->ifa_addr)->sin_addr
)
);
        strncpy(lpm->net_conf.if_names[i],
                ifp->ifa_name,
                sizeof lpm->net_conf.if_names[i]);
        strncpy(lpm->net_conf.if_addr[i],
                inet_ntoa(
                    ((struct sockaddr_in *)ifp->ifa_addr)->sin_addr
                ),
                sizeof lpm->net_conf.if_addr[i]);
        ++i;
    }

    freeifaddrs(ifp);

    return i;
}
#endif

int retrieve_if(LinphoneMain *lpm)
{
	int sock,err,if_count,i,j=0,lo=0;
	struct ifconf netconf;
	char buffer[32*MAX_IF];
	
	netconf.ifc_len=32*MAX_IF;
	netconf.ifc_buf=buffer;
	sock=socket(PF_INET, SOCK_DGRAM,0);
	err=ioctl(sock,SIOCGIFCONF,&netconf);
	if (err<0) printf("Error in ioctl: %i.\n",errno);
	close(sock);
	
	if_count=netconf.ifc_len/32;
	printf("Found %i interfaces.\n",if_count);
	for (i=0;i<if_count;i++)
	{
		if (strcmp(netconf.ifc_req[i].ifr_name,"lo")==0) lo=1;

		strncpy(lpm->net_conf.if_names[j],netconf.ifc_req[i].ifr_name,20);
		strncpy(lpm->net_conf.if_addr[j],inet_ntoa(((struct sockaddr_in*)(&netconf.ifc_req[i].ifr_addr))->sin_addr),20);
		j++;
	
	}
	if (lo==0) return(-1);
	return(if_count);
}

static char lock_name[80];
static char lock_set=0;

/* put a lock file in /tmp. this is called when linphone runs as a daemon*/
int set_lock_file()
{
	FILE *lockfile;
	
	snprintf(lock_name,80,"/tmp/linphone.%i",getuid());
	lockfile=fopen(lock_name,"w");
	if (lockfile==NULL)
	{
		printf("Failed to create lock file.\n");
		return(-1);
	}
	fprintf(lockfile,"%i",getpid());
	fclose(lockfile);
	lock_set=1;
	return(0);
}

/* looks if there is a lock file. If presents return its content (the pid of the already running linphone), if not found, returns -1*/
int get_lock_file()
{
	
	int pid;
	FILE *lockfile;
	
	snprintf(lock_name,80,"/tmp/linphone.%i",getuid());
	lockfile=fopen(lock_name,"r");
	if (lockfile==NULL)
		return(-1);
	fscanf(lockfile,"%i",&pid);
	fclose(lockfile);
	return(pid);
}

/* remove the lock file if it was set*/
int remove_lock_file()
{
	int err=0;
	if (lock_set)
	{
		err=unlink(lock_name);
		lock_set=0;
	}
	return(err);
}

static int esd_killed=0;
static int artsd_killed=0;

int kill_sound_daemons()
{
	char command[100];
	int err;
	FILE *out;
	pid_t pid;
	
	/* try to kill all artsd*/
	snprintf(command,100,"ps -u %s |grep artsd",getenv("LOGNAME"));
	out=popen(command,"r");
	if (out!=NULL)
	{
		do{
			err=fscanf(out,"%i",&pid);
			if (err==1) {
				kill(pid,SIGINT);
				artsd_killed=1;
			}
		}while(err==1);
		pclose(out);
	}
	/* do the same with esd*/
	snprintf(command,100,"ps -u %s |grep esd",getenv("LOGNAME"));
	out=popen(command,"r");
	if (out!=NULL)
	{
		do{
			err=fscanf(out,"%i",&pid);
			if (err==1) {
				kill(pid,SIGINT);
				esd_killed=1;
			}
		}while(err==1);
		pclose(out);
	}
	return(0);
}

void restore_sound_daemons()
{
	pid_t pid;
	if (esd_killed){
		esd_killed=0;
		pid=fork();
		if (pid==0){
			/* new process */
			execl("esd",NULL);
		}
	}
	if (artsd_killed){
		artsd_killed=0;
		pid=fork();
		if (pid==0){
			/* new process */
			execl("artsd",NULL);
		}
	}
}