/*
linphone
Copyright (C) 2000  Simon MORLAT (simon.morlat@free.fr)

callbacks.c -- gtk callbacks, and osipua callbacks.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <applet-widget.h>
#include <string.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "state-machine.h"
#include "sip_handlers.h"
#include "LinphoneMain.h"
#include "gui_utils.h"
#include  "propertybox.h"
#include "../pixmaps/linphone2.xpm"
#include "../osipua/src/utils.h"

#ifdef HAVE_PANEL_PIXEL_SIZE

#endif



/* widgets*/
extern GtkWidget *applet;
extern GtkWidget *main_window;
extern GtkWidget *appbar1;
extern GtkWidget *addressentry;
LPPropertyBox *prop;

/* LinphoneMain object*/
extern LinphoneMain linpmain;


/* messages strings */
extern gchar *ready;
extern gchar *end;
extern gchar *contacting;
extern gchar *connected;
extern gchar *cancel;
extern gchar *contacted;

#ifndef G_THREADS_ENABLED
#error "pas de threads dans la glib!"
#endif

int state=LP_INIT;  /* this integer represents the state machine: it MUST only be changed using a mutex or something
like*/
G_LOCK_DEFINE_STATIC(state);
char *rem_url=NULL;
GList *if_list;
static unsigned char selected_row=0;
RegistrationCtxt *regctx=NULL;
transaction_t *current_trn;


void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (main_window!=NULL)
	{
		gnome_entry_save_history (GNOME_ENTRY (addressentry));
	}
	else printf("Error: main_window=NULL.\n");
	gui_shutdown(&linpmain);
	
}

void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *about2;
  about2 = create_about2 ();
  gtk_widget_show (about2);
}


void
on_greenbutton_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *appbar1=(GtkWidget*)user_data;
	gchar *url, *barmsg,*remaddr,*remport,*payload;
	int err;
	BodyContext *sdpctx;
	sdp_t *localsdp,*remsdp;
	int pos;
	char *media_type;
	char *localport;
	gint jitt_comp;

	G_LOCK(state);
	printf("state=%i\n",state);
	
	if (state==LP_INVITED)
	{
		/*stop ringing */
		if (linpmain.ringstream!=NULL) {
			ring_stop(linpmain.ringstream);
			linpmain.ringstream=NULL;
		}
		/* sends a 200 OK */
		osip_dialog_accept_invite(linpmain.call,current_trn);
		gnome_appbar_push(GNOME_APPBAR(appbar1),connected);
		gtk_entry_set_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(addressentry))),rem_url);
		state=LP_INVITED_CONFIRMED;
		
		sdpctx=osip_dialog_get_body_context(linpmain.call,"application/sdp",0);
		localsdp=SDP_CONTEXT(sdpctx)->localneg;
		if (localsdp==NULL){
			g_warning("Could not find local sdp.");
			goto end;
		}
		remsdp=SDP_CONTEXT(sdpctx)->remote;
		if (remsdp==NULL){
			g_warning("Could not find remote sdp.");
			goto end;
		}
		remaddr=sdp_c_addr_get(remsdp,-1,0);
		if (remaddr==NULL){
			remaddr=sdp_c_addr_get(remsdp,0,0);
		}

		pos=0;
		localport=sdp_m_port_get(localsdp,pos);
		media_type = sdp_m_media_get(localsdp,pos);
		while ((localport!=NULL&&media_type!=NULL)&&
		       (0==strncmp(localport,"0", 1)||0!=strcmp(media_type,"audio")))
		  { /* If we have refused some media lines, the port is set to 0 */
		    pos++;
		    media_type = sdp_m_media_get(localsdp,pos);
		    localport=sdp_m_port_get(localsdp,pos);
		  }
		remport = sdp_m_port_get(remsdp,pos);
		payload = sdp_m_payload_get(localsdp,pos,0);
		if (remport!=NULL && media_type!=NULL) /* if codec has been found */
		{
			if (try_open_dsp()){
				/* adjust rtp jitter compensation. It must be at least the latency of the sound card */
				jitt_comp=MAX(linpmain.sound_conf.latency,linpmain.rtp_conf.jitt_comp);
				linpmain.audiostream=audio_stream_start(linpmain.rtp_conf.audio_rtp_port,remaddr,atoi(remport),atoi(payload),jitt_comp);
			}
		}
		else printf("No accepted audio codec found. But session is established\n");
	}
	else if (state==LP_INIT)
	{
		char *correct_url;
		/* sends an invite */
		url=gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(addressentry))));
		if (url==NULL)
		{
			//printf("gtk_entry_get_text() returned NULL. This is not normal if some text is present.\n");
			gnome_app_warning(GNOME_APP(main_window),_("Bad formuled sip address."));
			goto end;
		}
		correct_url=check_sipurl(url);
		if (correct_url==NULL)
		{
			gnome_app_warning(GNOME_APP(main_window),_("Bad formuled sip address."));
			goto end;
		}
		if (correct_url!=url) gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(addressentry))),correct_url);
		url=correct_url;
	
		barmsg=g_malloc(strlen(url)+strlen(contacting)+2);
		sprintf(barmsg,"%s %s",contacting,url);
		gnome_appbar_push(GNOME_APPBAR(appbar1),barmsg);
		
		linpmain.call=osip_dialog_new(linpmain.ua);
		err=osip_dialog_invite(linpmain.call,url,"application/sdp");
		if (err!=0)
		{
			printf("error in invite: %s (%i)\n",strerror(err),err);
			//gnome_app_warning(GNOME_APP(main_window),strerror(err));
			gnome_appbar_push(GNOME_APPBAR(appbar1),ready);
			state=LP_INIT;
		}	
		else state=LP_INVITE;
	}
end:
	printf("invite state=%i\n",state);
	G_UNLOCK(state);
}


void
on_redbutton_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
    GtkWidget *appbar1=(GtkWidget*)user_data;

    G_LOCK(state);
    printf("state end=%i\n",state);
    if (state==LP_INVITED )
    {
       /*stop ringing*/
		if (linpmain.ringstream!=NULL) {
			ring_stop(linpmain.ringstream);
			linpmain.ringstream=NULL;
		}
		/* decline the invitation */
		osip_dialog_bye(linpmain.call);
		gnome_appbar_push(GNOME_APPBAR(appbar1),end);
		state=LP_INIT;
    }
	else if ((state==LP_INVITED_CONFIRMED) || (state==LP_INVITE_CONFIRMED))
	{
    		 /* sends a bye, the call is terminated */
		if (linpmain.audiostream!=NULL) {
			audio_stream_stop(linpmain.audiostream);
			linpmain.audiostream=NULL;
			restore_sound_daemons();
		}
		osip_dialog_bye(linpmain.call);
		gnome_appbar_push(GNOME_APPBAR(appbar1),end);
		state=LP_INIT;
	}
	else if (state==LP_INVITE)
	{
     /*sends a cancel or a bye ???*/
		if (linpmain.ringstream!=NULL) {
			ring_stop(linpmain.ringstream);
			linpmain.ringstream=NULL;
		}
		osip_dialog_bye(linpmain.call);
		gnome_appbar_push(GNOME_APPBAR(appbar1),cancel);
		state=LP_INIT;
	}
	linpmain.call=NULL;
	G_UNLOCK(state);
	
	return;
}

int invite_accepted_cb(OsipDialog *call,transaction_t *trn, sip_t *msg,void*p2)
{
	char *remaddr,*remport,*payload;
	SdpContext *sdpctx;
	sdp_t *localsdp,*remsdp;
	gint jitt_comp;
	gint err;

	G_LOCK(state);
	gdk_threads_enter();
	main_window_show();
	gnome_appbar_push(GNOME_APPBAR(appbar1),connected);
	gdk_threads_leave();
	state=LP_INVITE_CONFIRMED;
	if (linpmain.ringstream!=NULL){
		ring_stop(linpmain.ringstream);
		linpmain.ringstream=NULL;
	}
	G_UNLOCK(state);
	
	sdpctx=SDP_CONTEXT(osip_dialog_get_body_context(linpmain.call,"application/sdp",0));
	localsdp=sdpctx->localgen;
	remsdp=sdpctx->remote;
	remaddr=sdp_c_addr_get(remsdp,-1,0);
	if (remaddr==NULL){
		remaddr=sdp_c_addr_get(remsdp,-1,0);
	}
	remport=sdp_m_port_get(remsdp,0);
	payload=sdp_m_payload_get(remsdp,0,0);
	
	gdk_threads_enter();
	err=try_open_dsp();
	gdk_threads_leave();
	if (err>0){
		/* adjust rtp jitter compensation. It must be at least the latency of the sound card */
		jitt_comp=MAX(linpmain.sound_conf.latency,linpmain.rtp_conf.jitt_comp);
		linpmain.audiostream=audio_stream_start(linpmain.rtp_conf.audio_rtp_port,remaddr,atoi(remport),atoi(payload),jitt_comp);
	}
	return 0;
}

int bye_cb(OsipDialog *call,transaction_t *trb,sip_t *msg,void*p2)
{
	/*stop ringing if necessary*/
	if (linpmain.ringstream!=NULL) {
		ring_stop(linpmain.ringstream);
		linpmain.ringstream=NULL;
	}
	if (linpmain.audiostream!=NULL) {
		audio_stream_stop(linpmain.audiostream);
		linpmain.audiostream=NULL;
	}
	G_LOCK(state);
	gdk_threads_enter();
	main_window_show();
	gnome_appbar_push(GNOME_APPBAR(appbar1),end);
	gdk_threads_leave();
	state=LP_INIT;
	linpmain.call=NULL;
	printf("bye handler: state=%i\n",state);
	G_UNLOCK(state);
	restore_sound_daemons();
	return 0;
}

/* osipua call us in this callback function when an invite arrives*/
int invite_cb(OsipDialog *call,transaction_t *trn, sip_t *sipmsg,void *p2)
{
	from_t *from=sipmsg->from;
	char *barmesg;
	int addrlen;
	BodyContext *sdpctx;

	/* first check if we can answer successfully to this invite */
	G_LOCK(state);
	sdpctx=osip_dialog_get_body_context(call,"application/sdp",0);
	/* get the result of the negociation */
	switch(SDP_CONTEXT(sdpctx)->negoc_status)
	{
		case -1:	
			printf("Error during sdp negociation. Cannot accept incoming call.\n");
			osip_dialog_respond(call,trn,606);
		break;
		case 200:
			linpmain.call=call;
			if (rem_url!=NULL) g_free(rem_url);
			addrlen=9+strlen(from->url->username)+strlen(from->url->host);
			if (from->displayname!=NULL) addrlen+=strlen( from->displayname);
			rem_url=g_malloc(addrlen);
			if (from->displayname!=NULL)
				sprintf(rem_url,"%s <sip:%s@%s>", from->displayname, from->url->username,  from->url->host);
			else sprintf(rem_url,"<sip:%s@%s>", from->url->username,  from->url->host);
			barmesg=g_malloc(strlen(rem_url)+strlen(contacted)+4);
			sprintf(barmesg,"%s %s.",rem_url,contacted);
			gdk_threads_enter();
			main_window_show();
			gnome_appbar_push(GNOME_APPBAR(appbar1),barmesg);
			gdk_threads_leave();
			state=LP_INVITED;
			/* play the ring */
			/* see if dsp is openable*/
			gdk_threads_enter();
			if (try_open_dsp()>0)
				linpmain.ringstream=ring_start(linpmain.sound_conf.local_ring,2);
			gdk_threads_leave();
			
			current_trn=trn;
			printf("invite handler done.\n");
			g_free(barmesg);
		break;
		default:
			osip_dialog_respond(call,trn,SDP_CONTEXT(sdpctx)->negoc_status);
	}
	G_UNLOCK(state);
	return 0;
}

int informative_cb( OsipDialog *call,transaction_t *trn, sip_t *msg,void *p2)
{
	int err=((int*)p2)[0];
	contact_t *contact;
	char *ctt;
			
	gdk_threads_enter();
	switch(err)
	{
		case 100:
			break;
		case 180:
			if (try_open_dsp()>0)
				linpmain.ringstream=ring_start(linpmain.sound_conf.remote_ring,2);
			break;
		case 200:   /* for the moment only REGISTER request followed by 200 OK are said to the user*/
			 if (regctx->expires) gnome_appbar_push(GNOME_APPBAR(appbar1),_("Registration successfull."));
			 else  gnome_appbar_push(GNOME_APPBAR(appbar1),_("Unregistration successfull."));
		break;
		case 380:
			msg_getcontact(msg,0,&contact);
			if (contact!=NULL)
			{			
				contact_2char(contact,&ctt);
			}else ctt=sgetcopy(_("(No contact information !)"));
			alt_ressource_display(ctt);
			sfree(ctt);
			//gtk_main_iteration_do(FALSE);
		break;			
		default:
			printf("unsupported informative message.\n");
	}
	gdk_threads_leave();
	return 0;
}

int faillure_cb(OsipDialog *call,transaction_t *trn, sip_t *msg,void *p2)
{
	int err=((int*)p2)[0];
	char *reason;
	header_t *retry;
	char *umsg;
	char *msg486=_("User is busy.");
	char *msg480=_("User is temporarily unavaillable.");
	char *retrymsg=_("%s. Retry after %i minute(s).");
	char *msg600=_("User does not want to be disturbed.");
	char* tmpmsg=msg486;
	
	G_LOCK(state);
	gdk_threads_enter();
	main_window_show();
	switch(err)
	{
		case 404:
			gnome_appbar_push(GNOME_APPBAR(appbar1),_("User cannot be found at given address."));
			state=LP_INIT;
		break;
		case 415:
			gnome_appbar_push(GNOME_APPBAR(appbar1),_("Remote user cannot support any of proposed codecs."));
		break;
		case 480:
			tmpmsg=msg480;
		case 486:
			msg_header_getbyname(msg,"retry-after",0,&retry);
			if (retry!=NULL)
			{
				umsg=g_malloc(strlen(tmpmsg)+strlen(retrymsg)+13);
				sprintf(umsg,retrymsg,tmpmsg,atoi(retry->hvalue)/60);
				gnome_app_message(GNOME_APP(main_window),umsg);
				g_free(umsg);
			}		
			gnome_appbar_push(GNOME_APPBAR(appbar1),tmpmsg);
			state=LP_INIT;
		break;
		
		case 600:
			gnome_appbar_push(GNOME_APPBAR(appbar1),msg600);
			state=LP_INIT;
		break;
		case -110:  /* time out, call leg is lost */
			state=LP_INIT;
			gnome_appbar_push(GNOME_APPBAR(appbar1),_("Timeout."));
		break;
		case -111:
			gnome_appbar_push(GNOME_APPBAR(appbar1),_("Remote host was found but refused connection."));
			state=LP_INIT;
		break;
		
		default:
			if (err>0)
			{
				reason=msg_getreason(err);
				gnome_appbar_push(GNOME_APPBAR(appbar1),reason);
				free(reason);
			}
			else printf("faillure_cb: %s\n",strerror(-err));
			if (linpmain.ringstream!=NULL) {
				ring_stop(linpmain.ringstream);
				linpmain.ringstream=NULL;
			}
			
	}
	state=LP_INIT;
	linpmain.call=NULL;
	gdk_threads_leave();
	G_UNLOCK(state);
	printf("error handler done.\n");
	return 0;
}

void on_propertybox1_apply (GtkButton       *button,
                                        gint pagenum)
{	
	switch(pagenum)
	{
	case 0:
		net_section_apply(&prop->net,&linpmain);
		break;
	case 1:
		rtp_section_apply(&prop->rtp,&linpmain);
		break;
	case 2:
		sip_section_apply(&prop->sip,&linpmain);
		break;
	case 4:
		sound_section_apply(&prop->sound,&linpmain);
  		break;
  	case 3:
  	 	codec_section_apply(&prop->codec,&linpmain);
  		break;
  	}
}

gint
on_prop1_close                         (GnomeDialog     *gnomedialog,
                                        gpointer         user_data)
{	

	gnome_appbar_clear_stack( GNOME_APPBAR(appbar1));
	return(FALSE);
}

void
on_parametres1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	prop=lp_property_box_new();
	lp_property_box_fill(prop,&linpmain);
  	
	gtk_widget_show(prop->prop);
}


void
on_killesd_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
	
	kill_sound_daemons();
}

void kill_esd_answer(gint reply, gpointer data)
{
	if (!reply)
	{
		printf("Killing sound daemons.\n");
		kill_sound_daemons();
	}
}

gboolean
on_hscale1_button_release_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
	return(TRUE);
}

void
on_list1_select_child                  (GtkList         *list,
                                        GtkWidget       *widget,
                                        gpointer         user_data)
{
	
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_codec_changed                       (GtkEditable     *editable,
                                        gpointer         user_data)
{
	/*
	gchar *sel_cod;
	sel_cod=gtk_editable_get_chars(editable,0,-1);
	if (strcmp(sel_cod,"GSM (13 kbit/s)")==0) temp_config.profile=3;
	else if (strcmp(sel_cod,"LPC10-15 (2.4 kbit/s)")==0) temp_config.profile=4;
	printf("Profile selection changed to %s : %i.\n",sel_cod,temp_config.profile);
	g_free(sel_cod);
	*/
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_sounddriver_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	/*
	gchar *sel_sound;
	
	sel_sound=gtk_editable_get_chars(editable,0,-1);
	if (strcmp(sel_sound,"OSS")==0) temp_config.sound_driver=0;
	else if (strcmp(sel_sound,"ALSA")==0) temp_config.sound_driver=1;
	printf("Sound driver selection changed to %s : %i.\n",sel_sound,temp_config.sound_driver);
	g_free(sel_sound);
	*/
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_user_manual1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	static GnomeHelpMenuEntry help_ref = { "linphone", "manual/index.html" };
	gnome_help_display(NULL,&help_ref);
}


gboolean
on_play_vol_button_release_event       (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	int vol;
	vol=(gtk_range_get_adjustment(GTK_RANGE(widget)))->value;
	ms_oss_set_plevel(&oss_mgr,linpmain.sound_conf.dev_id,vol);
	linpmain.sound_conf.play_lev=vol;
  return FALSE;
}


gboolean
on_rec_vol_button_release_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	int vol;
	vol=(gtk_range_get_adjustment(GTK_RANGE(widget)))->value;
	ms_oss_set_rlevel(&oss_mgr,linpmain.sound_conf.dev_id,vol);
	linpmain.sound_conf.rec_lev=vol;
	return FALSE;
}


void
on_prop1_help                          (GnomePropertyBox *gnomepropertybox,
                                        gint             arg1,
                                        gpointer         user_data)
{
	static GnomeHelpMenuEntry ref = { "linphone", "manual/params.html" };
	gnome_help_display(NULL,&ref);
}


void
on_audioport_changed                   (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
	return;
}


void
on_sipport_changed                     (GtkEditable     *editable,
                                        gpointer         user_data)
{

	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
	return;
}


void
on_source_changed                      (GtkEditable     *editable,
                                        gpointer         user_data)
{
	
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_fermer1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if ((linpmain.gui_conf.is_applet) ||( linpmain.gui_conf.is_daemon) )
	{
		gtk_widget_destroy(main_window);
	}
}


void
on_autokill_button1_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_app1_destroy                        (GtkObject       *object,
                                        gpointer         user_data)
{
  	if ((linpmain.gui_conf.is_applet) ||( linpmain.gui_conf.is_daemon) )
  	{
  		printf("Main window destroyed.\n");
  		main_window=NULL;
  	}
	else
	{
	
    	gnome_entry_save_history (GNOME_ENTRY (addressentry));
    	gui_shutdown(&linpmain);
    	
	}
}

/* this is the callback when the user clicks on the applet*/
gboolean on_applet_clicked(GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer ud)
{
	if (main_window==NULL)
	{
		main_window_show();
	}
	else
	{
		gtk_widget_destroy(main_window);
		main_window=NULL;
	}
	return(TRUE);
}


/*this is when the panel size changes*/
void applet_change_pixel_size(GtkWidget *w, int size, gpointer data)
{
	GtkWidget *pixmap,*button;
	
	pixmap=gtk_object_get_data(GTK_OBJECT(applet),"applet_pixmap");	
	button=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(applet),"applet_button");	
	if (button==NULL)
	{
		printf("Cannot find applet button\n");
		return;
	}
	if (pixmap!=NULL) gtk_widget_destroy(pixmap);
	pixmap = gnome_pixmap_new_from_xpm_d_at_size(linphone2_xpm,
							 size-4, size-4);
	gtk_object_set_data(GTK_OBJECT(applet),"applet_pixmap",pixmap);
  	gtk_widget_show(pixmap);
  	gtk_container_add(GTK_CONTAINER(button), pixmap);	
}

void
on_adresse_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *addbook;
	GtkWidget *item;
	GList *item_list=NULL,*elem=linpmain.gui_conf.address_list;
	GtkWidget *list;
	
	addbook=create_address_book();
	list=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_list");
	while(elem!=NULL)
	{
		item=gtk_list_item_new_with_label ((gchar*)(elem->data));
		item_list=g_list_append(item_list,item);
		gtk_widget_show(item);
		elem=g_list_next(elem);
	}
	gtk_list_append_items(GTK_LIST(list), item_list);
	gtk_widget_show(addbook);
}

void on_address_book_show(GtkWidget *widget,gpointer user_data)
{
	
}



void on_address_list_select_child(GtkList         *list,
                                        GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *entry;
	from_t *sipaddr;
	gchar *address;
	gchar *url;
	gint error;
	GtkWidget *addbook=(GtkWidget*)user_data;
	
	gtk_label_get(GTK_LABEL(GTK_BIN(widget)->child),&address);
	from_init(&sipaddr);
	error=from_parse(sipaddr,address);
	if (error!=0){
		printf("error parsing sip address\n");
		return;
	}
	
	entry=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"name_entry");
	gtk_entry_set_text(GTK_ENTRY(entry),sipaddr->displayname);
	sfree(sipaddr->displayname);
	sipaddr->displayname=NULL;
	from_2char(sipaddr,&url);
	entry=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_entry");
	gtk_entry_set_text(GTK_ENTRY(entry),url);
	from_free(sipaddr);
	sfree(sipaddr);
	sfree(url);
}

void
on_modify_address_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *addbook=(GtkWidget*)user_data;
	GtkWidget *list,*entry;
	gchar *address,*full_address,*displayname;
	GList *elem;
	gint selected_address;
	GList *item_list=NULL;
	GtkWidget *item;
	from_t *sipaddr;
	gint error;
	
	entry=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_entry");
	address=  gtk_editable_get_chars (GTK_EDITABLE(entry),0,-1);
	entry=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"name_entry");
	displayname=  gtk_editable_get_chars (GTK_EDITABLE(entry),0,-1);
	from_init(&sipaddr);
	error=from_parse(sipaddr,address);
	if (error!=0)
	{
		printf("Sip address is incorrect.\n");
		free(sipaddr);
		g_free(address);
		g_free(displayname);
		return;
	}
	sipaddr->displayname=sgetcopy(displayname);
	from_2char(sipaddr,&full_address);
	/* now update the address in the address list, and in the gui list */
	/* remove the actual address in the list */
	list=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_list");
	g_return_if_fail(list!=NULL);
	g_return_if_fail(GTK_LIST(list)->selection!=NULL);
	selected_address=g_list_index (GTK_LIST(list)->children,GTK_LIST(list)->selection->data);
	gtk_list_clear_items(GTK_LIST(list), selected_address,selected_address+1);
	elem= g_list_nth(linpmain.gui_conf.address_list,selected_address);
	if (elem==NULL)
	{
		printf("Error: cannot retrieve address\n");
		return;
	}
	g_free(elem->data);
	elem->data=full_address;
	/* then insert the new address in the gui list*/
	item=gtk_list_item_new_with_label (full_address);
	item_list=g_list_append(item_list,item);
	gtk_list_insert_items (GTK_LIST(list),item_list,selected_address);
	gtk_widget_show(item);
}

void on_add_address_clicked(GtkButton *button,gpointer user_data)
{
	gchar *address,*full_address,*displayname;
	GtkWidget *list,*entry;
	GtkWidget *addbook=(GtkWidget*)user_data;
	from_t *sipaddr;
	int error;
	GList *item_list=NULL;
	GtkWidget *item;
	
	
	entry=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_entry");
	address=  gtk_editable_get_chars (GTK_EDITABLE(entry),0,-1);
	entry=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"name_entry");
	displayname=  gtk_editable_get_chars (GTK_EDITABLE(entry),0,-1);
	from_init(&sipaddr);
	error=from_parse(sipaddr,address);
	if (error!=0)
	{
		printf("Sip address is incorrect.\n");
		free(sipaddr);
		g_free(address);
		g_free(displayname);
		return;
	}
	sipaddr->displayname=sgetcopy(displayname);
	from_2char(sipaddr,&full_address);
	
	linpmain.gui_conf.address_list=g_list_append( linpmain.gui_conf.address_list, (gpointer)full_address);
	item=gtk_list_item_new_with_label (full_address);
	item_list=g_list_append(item_list,item);
	list=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_list");
	gtk_list_append_items(GTK_LIST(list), item_list);
	gtk_widget_show(item);
	from_free(sipaddr);
	free(sipaddr);
	g_free(address);
	g_free(displayname);
}

void on_remove_address_clicked(GtkButton *button,gpointer user_data)
{
	GtkWidget *addbook=(GtkWidget*)user_data;
	GList *elem;
	GtkWidget *list;
	gint selected_address;
	
	list=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_list");
	g_return_if_fail(list!=NULL);
	g_return_if_fail(GTK_LIST(list)->selection!=NULL);
	selected_address=g_list_index (GTK_LIST(list)->children,GTK_LIST(list)->selection->data);
	gtk_list_clear_items(GTK_LIST(list), selected_address,selected_address+1);
	elem= g_list_nth(linpmain.gui_conf.address_list,selected_address);
	if (elem==NULL)
	{
		printf("Error: cannot retrieve address\n");
		return;
	}
	g_free(elem->data);
	linpmain.gui_conf.address_list=g_list_remove_link( linpmain.gui_conf.address_list,elem);
	g_list_free_1(elem);
	
}

void on_select_address_clicked(GtkButton *button,gpointer user_data)
{
	GtkWidget *addbook=(GtkWidget*)user_data;
	gchar *address;
	GtkWidget *list;
	gint selected_address;
	
	list=(GtkWidget*)gtk_object_get_data(GTK_OBJECT(addbook),"address_list");
	g_return_if_fail(list!=NULL);
	g_return_if_fail(GTK_LIST(list)->selection!=NULL);
	selected_address=g_list_index (GTK_LIST(list)->children,GTK_LIST(list)->selection->data);
 	address=(gchar*)g_list_nth_data(linpmain.gui_conf.address_list, selected_address);
	
	gtk_entry_set_text (GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(addressentry))),address);
	gtk_widget_destroy(addbook);
}




void
on_aucodec_up_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *clist=(GtkCList*)user_data;
	if (selected_row>0)
	{
		gtk_clist_row_move(clist,selected_row,selected_row-1);
		selected_row--;
		gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
	}
}


void
on_aucodec_down_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *clist=(GtkCList*)user_data;
	if (selected_row < clist->rows-1)
	{
		gtk_clist_row_move(clist,selected_row,selected_row+1);
		selected_row++;
		gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
	}
}


void
on_aucodec_enable_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *clist=(GtkCList*)user_data;
	gtk_clist_set_text(clist,selected_row,2,_("Enabled"));
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_aucodec_disable_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkCList *clist=(GtkCList*)user_data;
	gtk_clist_set_text(clist,selected_row,2,_("Disabled"));
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_au_codec_list_select_row            (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	MSFilterInfo *codec;
	selected_row=row;
	codec=gtk_clist_get_row_data(clist,row);
	g_return_if_fail(codec!=NULL);
	g_return_if_fail(user_data!=NULL);
	if (codec->description!=NULL){
		gtk_label_set_text(GTK_LABEL(user_data),codec->description);
	}else gtk_label_set_text(GTK_LABEL(user_data),_("No information availlable"));
}


void
on_au_codec_list_row_move              (GtkCList        *clist,
                                        gint             arg1,
                                        gint             arg2,
                                        gpointer         user_data)
{
	//gtk_clist_row_move(clist,arg1,arg2);
	//gnome_property_box_changed(GNOME_PROPERTY_BOX(prop1));
	//printf("moved!");
}




void
on_registrar_checked_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	gboolean state;
	
	state=gtk_toggle_button_get_active(togglebutton);
	sip_section_enable_registrar(&prop->sip,&linpmain, state);	
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_user_name_changed                   (GtkEditable     *editable,
                                        gpointer         user_data)
{
	GtkWidget *address_of_record=(GtkWidget *) user_data;
	gchar *tmp;
	/* address of record domain should usually be also the name of the registrar when it is used*/
	tmp=gtk_editable_get_chars(editable,0,-1);
	gtk_entry_set_text(GTK_ENTRY(address_of_record),tmp);
	g_free(tmp);
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_domain_name_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_registrar_addr_changed              (GtkEditable     *editable,
                                        gpointer         user_data)
{
	GtkWidget *address_of_record=(GtkWidget *) user_data;
	GtkWidget *username=prop->sip.username;
	url_t *url1,*url2;
	gchar *hostname,*addrrec,*tmp;
	gint err=0;
	/* address of record domain host part should usually be also the name of the registrar when it is used*/
	tmp=gtk_editable_get_chars(editable,0,-1);
	if ((tmp!=NULL) && (strlen(tmp)!=0))
	{
		url_init(&url1);
		err=url_parse(url1,tmp);
		g_free(tmp);
		if (err<0) {
			sfree(url1);
			return;
		}
	}
	else return;
	hostname=url1->host;
	addrrec=gtk_editable_get_chars(GTK_EDITABLE(address_of_record),0,-1);
	url_init(&url2);
	if ((addrrec!=NULL) && (strlen(addrrec)!=0)){
		err=url_parse(url2,addrrec);
		//printf("address of record parsed with err=%i\n",err);
		if (err<0) {url2->host=NULL;url2->username=NULL;};
	}
	if (url2->host!=NULL) sfree(url2->host);
	if ((hostname!=NULL) && (strlen(hostname)!=0)) url2->host=sgetcopy(hostname);
	else url2->host=gtk_editable_get_chars(GTK_EDITABLE(prop->sip.hostname),0,-1);
	//printf ("url2->host=%s\n",url2->host);
	if (url2->username==NULL) url2->username=gtk_editable_get_chars(GTK_EDITABLE(username),0,-1);
	url_setscheme(url2,sgetcopy("sip"));
	url_2char(url2,&tmp) ;
	if (tmp!=NULL){
		//printf("New address of record is %s\n",tmp);
		gtk_entry_set_text(GTK_ENTRY(address_of_record),tmp);
		sfree(tmp);
	}
	url_free(url1);
	sfree(url1);
    url_free(url2);
    sfree(url2);
	gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}

/* send a REGISTER request to the default registrar*/
gint do_registration(gpointer data)
{
	/* prevent from emmitting registration for address 127.0.0.1...*/
	if (strcmp(linpmain.net_conf.local_addr,"127.0.0.1")!=0)
	{
		OsipDialog *call=osip_dialog_new(linpmain.ua);
		if (regctx!=NULL) registration_ctxt_destroy(regctx);
		regctx=registration_ctxt_new();		
		registration_ctxt_set_registrar(regctx,linpmain.sip_conf.reg_conf.registrar);
		registration_ctxt_set_password(regctx, linpmain.sip_conf.reg_conf.passwd);
		registration_ctxt_set_address_of_record(regctx, linpmain.sip_conf.reg_conf.addr_of_rec);
		registration_ctxt_set_expires(regctx,linpmain.sip_conf.reg_conf.expires);
		
		if ((linpmain.sip_conf.reg_conf.as_redirect) && (!linpmain.sip_conf.reg_conf.as_proxy))
			registration_ctxt_set_action(regctx,REGISTER_ACTION_REDIRECT);
		else if ((linpmain.sip_conf.reg_conf.as_proxy) && (!linpmain.sip_conf.reg_conf.as_redirect))
			 registration_ctxt_set_action(regctx,REGISTER_ACTION_PROXY);
		else registration_ctxt_set_action(regctx,REGISTER_ACTION_ANY);
		osip_dialog_register(call,regctx);
		/* we don't want more from the dialog:*/
		osip_dialog_release(call);
		printf("Registration done.\n");
	}
	else printf("Registrations with local loopback address are forbidden in linphone.\n");
	return TRUE;
}
void
on_reg_passwd_changed                  (GtkEditable     *editable,
                                        gpointer         user_data)
{
   gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_redirect_button_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
   gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_proxy_button_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}


void
on_obproxy_button_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
   gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}



void
on_showmore_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	GtkWidget *optioncontrols=(GtkWidget*)user_data;
	gint state;
	state=gtk_toggle_button_get_active(togglebutton);
	if (state) gtk_widget_show(optioncontrols);
	else gtk_widget_hide(optioncontrols);
}



void
on_alt_href_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar *url,*sipurl;
	GtkWidget *label;
	label=GTK_BIN(button)->child;
	gtk_label_get(GTK_LABEL(label),&url);
	sipurl=check_sipurl(url);
	if (sipurl==url)
	{ /* it was a sip url, so display it in the entry*/
		gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(addressentry))),url);
	}else sfree(sipurl);
}

void
on_alt_href_realize                    (GtkWidget       *widget,
                                        gpointer         user_data)
{
  GdkCursor *cursor = gdk_cursor_new(GDK_HAND2);
  gdk_window_set_cursor(widget->window, cursor);
  gdk_cursor_destroy(cursor);
}




void
on_address_of_record_changed           (GtkEditable     *editable,
                                        gpointer         user_data)
{
    gnome_property_box_changed(GNOME_PROPERTY_BOX(prop->prop));
}

char * return_audio_port(SdpContext *ctx,int i){
	char *port=smalloc(10);
	snprintf(port,10,"%i",linpmain.rtp_conf.audio_rtp_port);
	return port;
}

sdp_t * generate_sdp(SdpContext *ctx,sdp_t *template)
{
	MSCodecInfo *codec;
	GList *elem;
	gchar *rtpmap;

	sdp_m_media_add(template, sgetcopy("audio"),
		       int2str(linpmain.rtp_conf.audio_rtp_port), NULL, sgetcopy("RTP/AVP"));
	
	elem=linpmain.codecs_conf.audio_codecs;
	while(elem!=NULL)
	{
		codec=(MSCodecInfo*)(elem->data);
		if ((codec->is_selected)&&(codec->is_usable))
		{
			sdp_m_payload_add(template,0,int2str(codec->pt));
			rtpmap=smalloc(strlen(codec->description)+10);
			sprintf(rtpmap,"%i %s",codec->pt,codec->description);
			sdp_a_attribute_add(template,0, sgetcopy("rtpmap"), rtpmap);
		}
		elem=g_list_next(elem);
	}
	return template;
}

