/*
  The mediastreamer library aims at providing modular media processing and I/O
	for linphone, but also for any telephony application.
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org
  										
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



#include "msspeexdec.h"

extern MSFilter * ms_speex_nb_enc_new();
extern MSFilter * ms_speex_lbr_enc_new();
extern MSFilter * ms_speex_wb_enc_new();


MSCodecInfo speex_nb_info=
{
	{
		"Speex codec",
		0,
		MS_FILTER_AUDIO_CODEC,
		ms_speex_nb_dec_new,
		"A high quality codec from Jean Marc Valin and David Rowe. "
		"This is the normal bit rate mode of the codec."
	},
	ms_speex_nb_enc_new,
	ms_speex_nb_dec_new,
	0,
	0,
	15000, /* bitrate */
	8000,	/* sampling frequency */
	110,		/* payload type */
	"set at init",
	1,
	1
};

MSCodecInfo speex_lbr_info=
{
	{
		"Speex codec (lbr)",
		0,
		MS_FILTER_AUDIO_CODEC,
		ms_speex_nb_dec_new,
		"A high quality codec from Jean Marc Valin and David Rowe. "
		"This is the low bit rate mode of the codec. This is the best codec for 28 or 56 kbit/s modems."
	},
	ms_speex_nb_enc_new,
	ms_speex_nb_dec_new,
	0,
	0,
	8000, /* bitrate */
	8000,	/* sampling frequency */
	111,		/* payload type */
	"set at init",
	1,
	1
};



void ms_speex_codec_init()
{
	speex_lbr_info.fr_size=speex_nb_mode.frame_size*2;
	speex_lbr_info.dt_size=(speex_nb_mode.frame_size*2*8000)/128000;
	speex_lbr_info.description=
		g_strdup_printf("speex_lbr-%i/8000/1",speex_nb_mode.bitstream_version);
	speex_nb_info.fr_size=speex_nb_mode.frame_size*2;
	speex_nb_info.dt_size=(speex_nb_mode.frame_size*2*speex_nb_mode.bitrate)/128000;
	speex_nb_info.description=
		g_strdup_printf("speex-%i/8000/1",speex_nb_mode.bitstream_version);
	ms_filter_register(MS_FILTER_INFO(&speex_nb_info));
	ms_filter_register(MS_FILTER_INFO(&speex_lbr_info));
}

gchar * g_module_check_init(GModule *module)
{
	ms_speex_codec_init();
	
	return NULL;
}

static MSSpeexDecClass * ms_speexwb_dec_class=NULL;
static MSSpeexDecClass * ms_speexnb_dec_class=NULL;

MSFilter * ms_speex_dec_new(SpeexMode *mode)
{
	MSSpeexDec *obj=g_new(MSSpeexDec,1);
	if (mode==&speex_nb_mode){
		if (ms_speexnb_dec_class==NULL){
			ms_speexnb_dec_class=g_new(MSSpeexDecClass,1);
			ms_speex_dec_class_init(ms_speexnb_dec_class,mode);
		}
		MS_FILTER(obj)->klass=MS_FILTER_CLASS(ms_speexnb_dec_class);
	}else
	{
		if (ms_speexwb_dec_class==NULL){
			ms_speexwb_dec_class=g_new(MSSpeexDecClass,1);
			ms_speex_dec_class_init(ms_speexwb_dec_class,mode);
		}
		MS_FILTER(obj)->klass=MS_FILTER_CLASS(ms_speexwb_dec_class);
	}
	
	ms_speex_dec_init(obj,mode);
	return MS_FILTER(obj);
}

/* mediastreamer compliant constructors */
MSFilter * ms_speex_nb_dec_new()
{
	return ms_speex_dec_new(&speex_nb_mode);
}

MSFilter * ms_speex_wb_dec_new()
{
	return ms_speex_dec_new(&speex_wb_mode);
}


void ms_speex_dec_init(MSSpeexDec *obj,SpeexMode *mode)
{
	int pf=1;
	ms_filter_init(MS_FILTER(obj));
	obj->speex_state=speex_decoder_init(mode);
	speex_bits_init(&obj->bits);
	/* enable the perceptual post filter */
	speex_decoder_ctl(obj->speex_state,SPEEX_SET_PF, &pf);
	
	MS_FILTER(obj)->outfifos=obj->outf;
	MS_FILTER(obj)->inqueues=obj->inq;
	obj->outf[0]=NULL;
	obj->inq[0]=NULL;
	obj->outbuf=g_malloc(sizeof(float)*MS_FILTER(obj)->klass->w_maxgran);
}

void ms_speex_dec_class_init(MSSpeexDecClass *klass, SpeexMode *mode)
{
	gint frame_size=0;
	
	ms_filter_class_init(MS_FILTER_CLASS(klass));
	frame_size=mode->frame_size;
	
	MS_FILTER_CLASS(klass)->process=(MSFilterProcessFunc)ms_speex_dec_process;
	MS_FILTER_CLASS(klass)->destroy=(MSFilterDestroyFunc)ms_speex_dec_destroy;
	ms_filter_class_set_name(MS_FILTER_CLASS(klass),"SpeexDecoder");
	MS_FILTER_CLASS(klass)->info=(MSFilterInfo*)&speex_nb_info;
	MS_FILTER_CLASS(klass)->max_foutputs=1;
	MS_FILTER_CLASS(klass)->max_qinputs=1;
	MS_FILTER_CLASS(klass)->w_maxgran=frame_size*2;
	MS_FILTER_CLASS(klass)->r_maxgran=frame_size;
	ms_trace("ms_speex_dec_class_init: w_maxgran is %i.",MS_FILTER_CLASS(klass)->r_maxgran);
}

void ms_speex_dec_uninit(MSSpeexDec *obj)
{
	speex_decoder_destroy(obj->speex_state);
	g_free(obj->outbuf);
}

void ms_speex_dec_destroy(MSSpeexDec *obj)
{
	ms_speex_dec_uninit(obj);
	g_free(obj);
}

void ms_speex_dec_process(MSSpeexDec *obj)
{
	MSFifo *outf=obj->outf[0];
	MSQueue *inq=obj->inq[0];
	gint16 *output;
	gint gran=MS_FILTER(obj)->klass->w_maxgran;
	gint i;
	MSMessage *m;
	
	g_return_if_fail(inq!=NULL);
	g_return_if_fail(outf!=NULL);
	
	m=ms_queue_get(inq);
	if (m->data!=NULL){
		speex_bits_reset(&obj->bits);
		speex_bits_read_from(&obj->bits,m->data,m->size);
		/* decode */
		speex_decode(obj->speex_state,&obj->bits,obj->outbuf,0);
	}else{
		/* we have a missing packet */
		speex_decode(obj->speex_state,&obj->bits,obj->outbuf,1);
	}
	ms_message_destroy(m);
	
	ms_fifo_get_write_ptr(outf,gran,(void**)&output);
	g_return_if_fail(output!=NULL);
	for (i=0;i<gran/2;i++) output[i]=obj->outbuf[i];
	
}
